/**
 *  Copyright 2009-2011 Matus Chochlik. Distributed under the Boost
 *  Software License, Version 1.0. (See accompanying file
 *  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 *
 *  WARNING: this file is here for development / testing purposes only
 *  and its contents and input / output can change witout any prior
 *  notice.
 */

//
// We want to use Lagoon's polymorphic factories
#define LAGOON_MT_WITH_MAKE_FACTORY 1
//
// We don't need to traverse through namespace members.
// Using this CT switch can greatly improve compile times
// and the resulting executable size if namespace member
// traversal is not needed
#define LAGOON_NO_NAMESPACE_MEMBERS 1


#include <mirror/mirror_base.hpp>
#include <mirror/pre_registered/full.hpp>
#include <lagoon/lagoon.hpp>
#include <lagoon/utils/script_factory.hpp>

#include <cassert>
#include <iostream>
#include <stdexcept>

#include "./test.hpp"

MIRROR_NAMESPACE_BEGIN
namespace aux {

} // namespace aux

MIRROR_NAMESPACE_END

int main(void)
{
	try
	{
		//
		using namespace lagoon;
		script_factory_builder builder;
		script_factory_input in;
		auto data = in.data();
		//
		auto meta_persons = reflected_class<std::vector<test::person> >();
		auto person_factory = meta_persons->make_factory(
			builder,
			raw_ptr(&data)
		);
		const std::string src = "{ \
			{mr, 'Johnny', 'B.', 'Goode', '1901-01-01', male, 70.4, 1.75e2}, \
			{mr, 'John', 'Doe', '1950-10-10', male, 75.9, 182} \
		}";
		//
		in.set(src);
		raw_ptr ppers = person_factory->new_();
		assert(ppers);
		std::vector<test::person>& pers = *raw_cast<std::vector<test::person>*>(ppers);
		std::cout << "There are " << pers.size() << " persons" << std::endl;
		meta_persons->delete_(ppers);
	}
	catch(std::exception const& error)
	{
		std::cerr << "Error: " << error.what() << std::endl;
	}
	//
	return 0;
}
